@@ -79,7 +79,6 @@ class Action(object):
         return f
 
     def popen(self, args, cwd=None, env=None, redirect=True, returnout=False):
-        logged_command = "%s$ %s" %(cwd, " ".join(map(str, args)))
         f = outpath = None
         resultjson = self.session.config.option.resultjson
         if resultjson or redirect:
@@ -93,7 +92,13 @@ class Action(object):
         if cwd is None:
             # XXX cwd = self.session.config.cwd
             cwd = py.path.local()
-        popen = self._popen(args, cwd, env=env, stdout=f, stderr=STDOUT)
+        try:
+            popen = self._popen(args, cwd, env=env,
+                                stdout=f, stderr=STDOUT)
+        except OSError:
+            self.report.error("invocation failed, args: %s, cwd: %s" %
+                              (args, cwd))
+            raise
         popen.outpath = outpath
         popen.args = [str(x) for x in args]
         popen.cwd = cwd
@@ -150,6 +155,7 @@ class Action(object):
         if env is None:
             env = os.environ.copy()
         return self.session.popen(args, shell=False, cwd=str(cwd),
+            universal_newlines=True,
             stdout=stdout, stderr=stderr, env=env)
 
 
@@ -221,6 +227,9 @@ class Reporter(object):
     def error(self, msg):
         self.logline("ERROR: " + msg, red=True)
 
+    def skip(self, msg):
+        self.logline("SKIPPED:" + msg, yellow=True)
+
     def logline(self, msg, **opts):
         self._reportedlines.append(msg)
         self.tw.line("%s" % msg, **opts)
@@ -409,14 +418,20 @@ class Session:
                 sdist_path = self._makesdist()
             except tox.exception.InvocationError:
                 v = sys.exc_info()[1]
-                self.report.error("FAIL could not package project")
+                self.report.error("FAIL could not package project - v = %r" %
+                    v)
                 return
             sdistfile = self.config.distshare.join(sdist_path.basename)
             if sdistfile != sdist_path:
                 self.report.info("copying new sdistfile to %r" %
                     str(sdistfile))
-                sdistfile.dirpath().ensure(dir=1)
-                sdist_path.copy(sdistfile)
+                try:
+                    sdistfile.dirpath().ensure(dir=1)
+                except py.error.Error:
+                    self.report.warning("could not copy distfile to %s" %
+                                        sdistfile.dirpath())
+                else:
+                    sdist_path.copy(sdistfile)
         return sdist_path
 
     def subcommand_test(self):
@@ -454,7 +469,14 @@ class Session:
         retcode = 0
         for venv in self.venvlist:
             status = venv.status
-            if status and status != "skipped tests":
+            if isinstance(status, tox.exception.InterpreterNotFound):
+                msg = "  %s: %s" %(venv.envconfig.envname, str(status))
+                if self.config.option.skip_missing_interpreters:
+                    self.report.skip(msg)
+                else:
+                    retcode = 1
+                    self.report.error(msg)
+            elif status and status != "skipped tests":
                 msg = "  %s: %s" %(venv.envconfig.envname, str(status))
                 self.report.error(msg)
                 retcode = 1
@@ -555,7 +577,7 @@ class Session:
             return candidates[0]
 
 
-_rex_getversion = py.std.re.compile("[\w_\-\+]+-(.*)(\.zip|\.tar.gz)")
+_rex_getversion = py.std.re.compile("[\w_\-\+\.]+-(.*)(\.zip|\.tar.gz)")
 def getversion(basename):
     m = _rex_getversion.match(basename)
     if m is None:
